<?php
/*
Plugin Name: SMS Vitrini Mesaj Paneli SMS
Plugin URI: http://www.smsvitrini.com
Description: WooCommerce plugin for sending SMS notifications to buyers and admins.
Version: 1.0.0
Author: SMS Vitrini
Author URI: http://www.smsvitrini.com/
License: GPL2
*/

/**
 * Copyright (c) 2017 Mesaj Paneli (email: destek@smsvitrini.com.tr). All rights reserved.
 *
 * Released under the GPL license
 * http://www.opensource.org/licenses/gpl-license.php
 *
 * This is an add-on for WordPress
 * http://wordpress.org/
 *
 * **********************************************************************
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 * **********************************************************************
 */

// Don't call the file directly
if (!defined('ABSPATH')) {
    exit;
}

// Lib Directory Path Constant
define('PLUGIN_CLASS_PATH', dirname(__FILE__) . '/includes');

// Require settings api
require_once PLUGIN_CLASS_PATH . '/class.settings-api.php';
require_once PLUGIN_CLASS_PATH . '/MesajPaneliOptions.php';
require_once PLUGIN_CLASS_PATH . '/MesajPaneliSMSGateway.php';

/**
 * Get SMS Settings Settings options value
 *
 * @param  string $option
 * @param  string $section
 * @param  string $default
 *
 * @return mixed
 */
function mesajpanelisms_get_option($option, $section, $default = '') {

    $options = get_option($section);

    if (isset($options[ $option ])) {
        return $options[ $option ];
    }

    return $default;
}

/**
 * MesajPaneli_Order_SMS class
 *
 * @class MesajPaneli_Order_SMS The class that holds the entire MesajPaneli_Order_SMS plugin
 */
class MesajPaneli_Order_SMS {

    /**
     * Constructor for the MesajPaneli_Order_SMS class
     *
     * Sets up all the appropriate hooks and actions
     * within our plugin.
     *
     * @uses is_admin()
     * @uses add_action()
     */
    public function __construct() {

        // Instantiate necessary class
        $this->instantiate();

        // Localize our plugin
        add_action('init', [ $this, 'localization_setup' ]);

        // Loads frontend scripts and styles
        add_action('admin_enqueue_scripts', [ $this, 'admin_enqueue_scripts' ]);

        if (mesajpanelisms_get_option('force_sms', 'mesajpanelisms_general', 'off') == 'off') {
            add_action('woocommerce_checkout_after_customer_details', [ $this, 'add_buyer_notification_field' ]);
            add_action('woocommerce_checkout_process', [ $this, 'add_buyer_notification_field_process' ]);
        }
        add_action('woocommerce_checkout_update_order_meta', [ $this, 'buyer_notification_update_order_meta' ]);
        add_action('woocommerce_admin_order_data_after_billing_address', [
            $this,
            'buyer_sms_notify_display_admin_order_meta'
        ], 10, 1);
        add_action('woocommerce_order_status_changed', [ $this, 'trigger_after_order_place' ], 10, 3);
        //add_filter( 'wc_order_statuses', array($this, 'filter_order_status'));

    }

    /**
     * Instantiate necessary Class
     *
     * @return void
     */
    function instantiate() {
        new MesajPaneliOptions();
        new MesajPaneliSMSGateway();
    }

    public static function init() {
        static $instance = false;

        if (!$instance) {
            $instance = new MesajPaneli_Order_SMS();
        }

        return $instance;
    }

    /**
     * Initialize plugin for localization
     *
     * @uses load_plugin_textdomain()
     */
    public function localization_setup() {
        load_plugin_textdomain('mesajpanelisms', false, dirname(plugin_basename(__FILE__)) . 'admin/languages/');
    }

    public function admin_enqueue_scripts() {

        wp_enqueue_style('admin-mesajpanelisms-styles', plugins_url('admin/css/admin.css', __FILE__), false, date('Ymd'));
        wp_enqueue_script('admin-mesajpanelisms-scripts', plugins_url('admin/js/admin.js', __FILE__), [ 'jquery' ], false, true);

        wp_localize_script('admin-mesajpanelisms-scripts', 'mesajpanelisms', [
            'ajaxurl' => admin_url('admin-ajax.php')
        ]);
    }

    /**
     * Add Buyer Notification field in checkout page
     */
    function add_buyer_notification_field() {

        if (mesajpanelisms_get_option('buyer_notification', 'mesajpanelisms_general', 'off') == 'off') {
            return;
        }

        $required      = (mesajpanelisms_get_option('force_buyer_notification', 'mesajpanelisms_general', 'no') == 'yes') ? true : false;
        $checkbox_text = mesajpanelisms_get_option('buyer_notification_text', 'mesajpanelisms_general', 'Send me order status notifications via sms');
        woocommerce_form_field('buyer_sms_notify', [
            'type'     => 'checkbox',
            'class'    => [ 'buyer-sms-notify form-row-wide' ],
            'label'    => __($checkbox_text, 'mesajpanelisms'),
            'required' => $required,
        ], 0);
    }

    /**
     * Add Buyer Notification field validation
     */
    function add_buyer_notification_field_process() {

        if (mesajpanelisms_get_option('force_buyer_notification', 'mesajpanelisms_general', 'no') == 'no') {
            return;
        }

        // Check if the field is set, if not then show an error message.
        if (!$_POST['buyer_sms_notify']) {
            wc_add_notice(__('<strong>Send Notification Via SMS</strong> must be required'), 'error');
        }
    }

    /**
     * Display Buyer notification in Order admin page
     *
     * @param  object $order
     *
     * @return void
     */
    function buyer_sms_notify_display_admin_order_meta($order) {
        $order_id          = trim(str_replace('#', '', $order->get_order_number()));
        $want_notification = get_post_meta($order_id, '_buyer_sms_notify', true);
        $display_info      = (isset($want_notification) && !empty($want_notification)) ? 'Evet' : 'Hayır';
        echo '<p><strong>' . __('Kullanıcı SMS ile bildirim istiyor.') . ':</strong> ' . $display_info . '</p>';
    }

    /**
     * Update Order buyer notify meta in checkout page
     *
     * @param  integer $order_id
     *
     * @return void
     */
    function buyer_notification_update_order_meta($order_id) {
        if (!empty($_POST['buyer_sms_notify'])) {
            update_post_meta($order_id, '_buyer_sms_notify', sanitize_text_field($_POST['buyer_sms_notify']));
        }
    }

    /**
     * Trigger when and order is placed
     *
     * @param  integer $order_id
     * @param  string  $old_status
     * @param  string  $new_status
     *
     * @return void
     */
    public function trigger_after_order_place($order_id, $old_status, $new_status) {
        $order = new WC_Order($order_id);
        $ywot  = get_post_custom($order_id);
        if (!$order_id) {
            return;
        }
        $admin_sms_data = $buyer_sms_data = [];

        $default_admin_sms_body = __('You have a new Order. The [order_id] is now [order_status]', 'mesajpanelisms');
        $default_buyer_sms_body = __('Thanks for purchasing. Your [order_id] is now [order_status]. Thank you', 'mesajpanelisms');
        $order_status_settings  = mesajpanelisms_get_option('order_status', 'mesajpanelisms_general', []);
        $admin_phone_number     = mesajpanelisms_get_option('sms_admin_phone', 'mesajpanelisms_message', '');
        $admin_sms_body         = mesajpanelisms_get_option('admin_sms_body', 'mesajpanelisms_message', $default_admin_sms_body);
        $buyer_sms_body         = mesajpanelisms_get_option('order_' . $new_status, 'mesajpanelisms_message', $default_buyer_sms_body);
        $active_gateway         = mesajpanelisms_get_option('sms_gateway', 'mesajpanelisms_gateway', 'MesajPaneli');
        $want_to_notify_buyer   = get_post_meta($order_id, '_buyer_sms_notify', true);

        if (count($order_status_settings) < 0 || empty($active_gateway)) {
            return true;
        }

        if (in_array($new_status, $order_status_settings) || $this->_isNewOrderActive()) {
            # send SMS to admin
            $notifyAdmin = [ 'on-hold', 'processing' ];

            if ($this->_isAdminSMSActive() && in_array($new_status, $notifyAdmin)) {
                $admin_numbers = explode(',', $admin_phone_number);

                foreach ($admin_numbers as $admin_number) {
                    $admin_sms_data['number']   = $admin_number;
                    $admin_sms_data['sms_body'] = $this->parse_sms_body($admin_sms_body, $new_status, $order->get_total(), $order_id, $ywot, $admin_number);
                    $admin_response             = MesajPaneliSMSGateway::MesajPaneliSendSMS($admin_sms_data);
                    if ($admin_response) {
                        $order->add_order_note(__('Admin numarasına SMS gönderilmiştir.', 'mesajpanelisms'));
                    }
                    else {
                        $order->add_order_note(__('Admin numarasına SMS gönderimi başarısız oldu.', 'mesajpanelisms'));
                    }
                }
            }
            # send SMS to buyer
            if (!$this->_isNewOrderActive() && $new_status == 'on-hold') {
                return true;
            }
            if (!$this->_isForceSMSActive() && !$want_to_notify_buyer) {
                return true;
            }

            $buyer_number               = get_post_meta($order_id, '_billing_phone', true);
            $buyer_sms_data['sms_body'] = $this->parse_sms_body($buyer_sms_body, $new_status, $order->get_total(), $order_id, $ywot, $buyer_number);

            if (!empty($buyer_sms_data['sms_body'])) {
                $buyer_sms_data['number'] = $buyer_number;
                $buyer_response           = MesajPaneliSMSGateway::MesajPaneliSendSMS($buyer_sms_data);
                if ($buyer_response) {
                    $order->add_order_note(__('Müşteriye SMS gönderilmiştir.', 'mesajpanelisms'));
                }
                else {
                    $order->add_order_note(__('Müşteriye SMS gönderimi başarısız oldu.', 'mesajpanelisms'));
                }
            }
        }
    }

    private function _isNewOrderActive() {
        return mesajpanelisms_get_option('enable_notification', 'mesajpanelisms_general', 'on') == 'on' ? true : false;
    }

    private function _isAdminSMSActive() {
        return mesajpanelisms_get_option('admin_notification', 'mesajpanelisms_general', 'on') == 'on' ? true : false;
    }

    public function parse_sms_body($content, $order_status, $order_amount, $order_id, $ywot, $number) {
        $order              = $order_id;
        $ywot_carrier_name  = (isset($ywot['ywot_carrier_name'][0])) ? $ywot['ywot_carrier_name'][0] : '';
        $ywot_pick_up_date  = (isset($ywot['ywot_pick_up_date'][0])) ? $ywot['ywot_pick_up_date'][0] : '';
        $ywot_tracking_code = (isset($ywot['ywot_tracking_code'][0])) ? $ywot['ywot_tracking_code'][0] : '';

        $order_total = $order_amount . ' ' . get_post_meta($order_id, '_order_currency', true);

        $filter_status = $this->filter_order_status($order_status);
        $find          = [
            '[order_id]',
            '[order_status]',
            '[ywot_carrier_name]',
            '[ywot_pick_up_date]',
            '[ywot_tracking_code]',
            '[buyer_number]',
        ];

        $replace = [
            $order,
            $filter_status,
            $ywot_carrier_name,
            $ywot_pick_up_date,
            $ywot_tracking_code,
            $number
        ];

        $body = str_replace($find, $replace, $content);

        return $body;
    }

    public function filter_order_status($new_status) {

        $status_posts = get_posts([
            'posts_per_page' => -1,
            'post_type'      => 'yith-wccos-ostatus',
            'post_status'    => 'publish'
        ]);
        foreach ($status_posts as $sp) {
            $statuses[ get_post_meta($sp->ID, 'slug', true) ] = $sp->post_title;
        }
        $default_statuses = [
            'pending'    => 'Ödeme Bekliyor',
            'processing' => 'İşleniyor',
            'on-hold'    => 'Beklemede',
            'completed'  => 'Tamamlandı',
            'cancelled'  => 'İptal Edildi',
            'refunded'   => 'İade Edildi',
            'failed'     => 'Başarısız'
        ];

        $order_statuses = (!empty($statuses)) ? array_merge($statuses, $default_statuses) : $default_statuses;

        $order_status = $order_statuses[ $new_status ];

        if (!$order_status) {
            return $new_status;
        }

        return $order_status;
    }

    private function _isForceSMSActive() {
        return mesajpanelisms_get_option('force_sms', 'mesajpanelisms_general', 'on') == 'on' ? true : false;
    }

}

add_action('plugins_loaded', 'load_MesajPaneli_order_sms');

function load_MesajPaneli_order_sms() {
    $mesajpanelisms = MesajPaneli_Order_SMS::init();
}
